#include <iostream>
#include <sstream>
#include "SimulatedRobot.h"
#include "SimulatedCamera.h"
#include "SimulatedSensors.h"
    
    //Konstruktor
    SimulatedRobot::SimulatedRobot( void )
    {
    	m_camera = new SimulatedCamera();
    	m_sensors = new SimulatedSensors();
    
    	if (!robotOK()) return;
    	
    	//verschiedene Initialisierungsversuche
    	sendCommand(" "); //alles anhalten
    	sendCommand("k"); //queue löschen
    	sendCommand("J"); //Job verarbeitung starten
    	
    	gettimeofday( &m_stopTime, 0);
    
    }
    
    //Desturktor
    SimulatedRobot::~SimulatedRobot( )
    {
        delete m_camera;
        delete m_sensors;
    }
    
    bool SimulatedRobot::robotOK( void ) const
    {
        return ( m_camera->cameraOK() && m_sensors->sensorsOK() );    
    }
    
    void SimulatedRobot::drive ( const float distance )
    {
        drive( distance, NORMAL_SPEED );
    }
    
    void SimulatedRobot::drive( const float distance, const unsigned int speed )
    {
        RobotInterface::drive( distance, speed ); //code vom interface aufrufen
         
         //hoppla, das ist etwas trickig, denn hiermit wird eine funktion aufgerufen die es nur in der simulierten Kamera gibt und nicht im Interface. Darum wird zuerst gecastet...
        ((SimulatedCamera*)m_camera)->move( distance );   
    }
    
    void SimulatedRobot::turn( const float angle )
    {
        turn( angle, NORMAL_SPEED );
    }
    
    void SimulatedRobot::turn( const float angle, const unsigned int speed )
    {
        RobotInterface::turn( angle, speed ); //code vom interface aufrufen
         
         //hoppla, das ist etwas trickig, denn hiermit wird eine funktion aufgerufen die es nur in der simulierten Kamera gibt und nicht im Interface. Darum wird zuerst gecastet...
        ((SimulatedCamera*)m_camera)->turn( angle );   
    }
    
    void SimulatedRobot::waitForStop( void ) const
    {
        //Es wird die geschaetzte Fahrtzeit abgewartet
        timeval now;
        gettimeofday( &now,0 );
        //cout << m_stopTime.tv_sec << ":" << m_stopTime.tv_usec << endl;
        //cout << now.tv_sec << ":" << now.tv_usec << endl;
        
        long diff = ( m_stopTime.tv_sec - now.tv_sec ) * 1000000;
        //cout << diff << endl;
        diff += m_stopTime.tv_usec;
        diff -= now.tv_usec;
        //cout << diff << endl;
        if (diff > 0 )
        {
        	    usleep( diff );
        }
    }
    
    void SimulatedRobot::sendCommand( const string s ) const
    {
    }
    
